!
! MODULE simulator 
!
! "THE BEER-WARE LICENSE":
! Alberto Ramos wrote this file. As long as you retain this 
! notice you can do whatever you want with this stuff. If we meet some 
! day, and you think this stuff is worth it, you can buy me a beer in 
! return. <alberto.ramos@cern.ch>
!
! file:    simulator.f90
! created: Tue Aug 21 18:43:39 2018
!                               
! EXTERNALLY ACCESSIBLE ROUTINES
!
! gen_series(dt,tau, mean): Fills dt(:) with autocorrelated data
!   with simple autocorrelation function e^{-t/tau} and
!   mean "mean"
!
! gen_series(dt, err, ti, tau, lam, mean): Fills dt(:) with
!   autocorrelated data with autocorrelation function
!   sum_{k} lam(k)e^{-t/tau(k)} and mean "mean". Returns in
!   err the exacty error and in ti the exact value of the
!   integrated autocorrelation time.
!

module simulator

  use ISO_FORTRAN_ENV, Only : error_unit, output_unit
  use numtypes
  use constants
  
  implicit none

  private

  logical, save :: first=.true.

  interface gen_series
     module procedure gen_series_multi, gen_series
  end interface

  public :: gen_series
  
contains

! *****************************
! *
  subroutine init_rndm()
! *
! *****************************
    integer, allocatable :: seed(:)
    integer              :: n, i
    
    if (first) then
       call random_seed(size=n)
       allocate(seed(n))
       forall (i=1:n) seed(i) = i
       call random_seed(put=seed)
       first = .false.
       deallocate(seed)
    end if
    
    return
  end subroutine init_rndm
  
!  *********************************************
!  *                                           *
  subroutine normal(X)
!  *                                           *
!  *********************************************
    Real (kind=DP), Intent(out) :: X(:)
    Real (kind=DP)              :: U1, U2
    
    integer            :: i, isz
    complex (kind=DPC) :: z
    real (kind=DP)     :: r

    call init_rndm()
    isz = int(size(X)/2)
    do i = 1, isz
       CALL Random_number(U1)
       CALL Random_number(U2)

       r        = Sqrt(-2.0_DP*Log(1.0_DP-U1))
       z        = exp(cmplx(0.0_DP,TWOPI_DP,kind=DP)*U2)
       X(2*i-1) = r*Real(z)
       X(2*i)   = r*Aimag(z)
    end do
    
    if (mod(size(X),2) == 1) Then
       CALL Random_number(U1)
       CALL Random_number(U2)

       r          = Sqrt(-2.0_DP*Log(1.0_DP-U1))
       z          = exp(cmplx(0.0_DP,TWOPI_DP,kind=DP)*U2)
       X(size(X)) = r*Real(z)
    end if
       
    return
  end subroutine normal

! *****************************
! *
  subroutine gen_series_multi(dt, err, ti, tau, lam, mean)
! *
! *****************************
    real (kind=DP), intent (inout)        :: dt(:)
    real (kind=DP), intent (out)          :: err, ti 
    real (kind=DP), intent (in)           :: tau(:), lam(:)
    real (kind=DP), intent (in), optional :: mean
    
    real (kind=DP) :: ap(size(tau)), eta(size(dt)), nu(size(dt)), dlt
    integer :: i, k, nc, nd

    nc = size(tau)
    nd = size(dt)
    where (tau.eq.0.0_DP)
       ap = 0.0_DP
    elsewhere
       ap = exp(-1.0_DP/tau)
    end where

    dt = mean
    do k = 1, nc
       call normal(eta)
       nu(1) = eta(1) 
       do i = 2, nd
          nu(i) = sqrt(1.0_DP-ap(k)**2)*eta(i) + ap(k)*nu(i-1)
       end do
       
       dt = dt + lam(k)*nu
    end do

    dlt = sum(lam**2)
    ti  = 0.5_DP + sum(lam**2/(1.0_DP/ap - 1.0_DP))/dlt
    err = sqrt(dlt/real(nd,kind=DP) * 2.0_DP*ti )
    
    return
  end subroutine gen_series_multi
    
! *****************************
! *
  subroutine gen_series(dt,tau, mean)
! *
! *****************************
    real (kind=DP), intent (inout)        :: dt(:)
    real (kind=DP), intent (in)           :: tau
    real (kind=DP), intent (in), optional :: mean
    
    real (kind=DP) :: ap, eta(size(dt))
    integer :: i
    
    if (tau.eq.0.0_DP) then
       ap = 0.0_DP
    else
       ap = exp(-1.0_DP/tau)
    end if
    call normal(eta)
    dt(1) = eta(1)
    do i = 2, size(dt)
       dt(i) = sqrt(1-ap**2)*eta(i) + ap*dt(i-1)
    end do

    if (present(mean)) dt = dt + mean
    
    return
  end subroutine gen_series

! *****************************
! *
  subroutine module_error(routine, msg)
! *
! *****************************	
    character (len=*), intent (in) :: routine, msg

    write(error_unit,*)'error '//trim(routine)//':'
    write(error_unit,'(5X,1A)')trim(msg)
    write(error_unit,*)
    
    error stop
  end subroutine module_error

end module simulator
 
