#!/usr/local/bin/zsh
#
# Plots a aderr history file
#
# Alberto Ramos <alberto.ramos@cern.ch>
#
# Usage: plot_hist.sh -i <input file> -o <dir>

help() {
    
    echo "Usage: plot_hist.sh -i <input file> -o <dir>"
    echo "   Creates dir/ID*-hist.pdf dir/ID*-auto.pdf dir/ID*-taui.pdf"
    exit -1
}

NOTIT=0
while [[ $# > 1 ]]
do
    key="$1"
    
    case $key in
	-i|--input)
	    INPUT=`realpath $2`
	    shift # past argument
	    ;;
	-o|--output)
	    DIR="$2"
	    shift # past argument
	    ;;
	--no-title)
	    NOTIT=1
	    shift
	    ;;
	*)
	    shift
	    ;;
    esac
done

if [[ -z $INPUT ]]; then
    help
fi
if [[ -z $DIR ]]; then
    help
fi

TMPMS=`mktemp`
TMPIW=`mktemp`
TMPRP=`mktemp`
TMPXP=`mktemp`
TMPTI=`mktemp`

NID=`cat $INPUT | grep 'Ensembles contributing' | awk '{print $2}'`
cat $INPUT | grep 'measurements in' | awk '{print $3}' > $TMPMS
cat $INPUT | grep 'Window:' | awk '{print $4}' > $TMPIW
cat $INPUT | grep 'measurements in' | awk '{print $6}' > $TMPRP
cat $INPUT | grep 'tauexp' | awk '{print $5}' > $TMPXP
cat $INPUT | grep 'Tauint' | awk '{print $4, $6}' > $TMPTI

for (( i=1; i <= $NID; i++ ))
do
    MS[i]=`sed -n $i\p $TMPMS`
    IW[i]=`sed -n $i\p $TMPIW`
    RP[i]=`sed -n $i\p $TMPRP`
    TX[i]=`sed -n $i\p $TMPXP`
    TI[i]=`sed -n $i\p $TMPTI | awk '{print $1}'`
    DT[i]=`sed -n $i\p $TMPTI | awk '{print $2}'`
done

offset="-2"
is="3"
MSTOT="0"

cp -f gnuplot-lua-tikz-common.tex gnuplot-lua-tikz.sty gnuplot-lua-tikz.tex t-gnuplot-lua-tikz.tex $DIR
for (( k=1; k <= $NID; k++ ))
do
    PREFIX="ID$k"
    GNP="$DIR/ID$k.gnp"

    for (( i=1; i <= $RP[k]; i++ ))
    do
	(( j = 4+$i ))
	IVREP[i]=`cat $INPUT | grep 'Replica vector' | sed -n $k\p | awk -v n=$j '{print $n}'`
	(( ie = $is + $IVREP[i]-1 ))
	#    sed -n "$is,$ie p" $INPUT > $GNP.$i
	grep -v '#' $INPUT | awk "NR>=$is && NR<=$ie {print NR+$offset, \$1}" > $GNP.$i
	
	echo $VAL
	(( VAL1 = $offset + $is ))
	(( VAL2 = $offset + $ie ))
	if [ $IW[k] -ge $VAL1 ] && [ $IW[k] -le $VAL2 ]; then
	    if [ "$TX[k]" = "0.0" ]; then
		RHOVAL="NaN"
	    else
		(( LN = $is + $IW[k] ))
		RHOVAL=`grep -v '#' $INPUT | sed -n $LN\p | awk '{print \$3}'`
	    fi
	fi

	(( is = $ie +1 ))
    done
    (( is = $ie + 5 ))
    (( offset = $offset - 4 - $MS[k] ))
    echo $TX[k], $RHOVAL

    if [ $IW[k] -lt 10 ]; then
	PW=40
    else
	(( PW=3*$IW[k] ))
    fi
    (( MS[k]=$MS[k]+2 ))
    
    if [ $NOTIT -eq 1 ]; then
	cat > $GNP <<DELIM
set colorsequence default
set style line 1 lw 2 lt 1 pt 7  ps 0.8
set style line 2 lw 2 lt 2 pt 7	 ps 0.8
set style line 3 lw 2 lt 7 pt 7	 ps 0.8
set style line 4 lw 2 lt -1 pt 7  ps 0.8
set style line 5 lw 2 lt 4 pt 7	 ps 0.8
set style line 6 lw 2 lt 18 pt 7 ps 0.8
set style line 60 lw 2 lt 0 pt 7 ps 0.8

set fit errorvariables
set term tikz standalone color font ",14"

file='$INPUT'

f(x) = x>$TX[k] ? $RHOVAL*exp(-(x - $IW[k])/$TX[k]) : NaN 

set key top right box spacing 1.2
set ylabel '$\rho(t)$'
set out '$PREFIX-auto.tex'
plot [:$PW] file i (( $k - 1 )) u 0:3:4 w yerr ls 1 t '$\rho(t)$'
set arrow from $IW[k]+0.5,GPVAL_Y_MIN to $IW[k]+0.5,GPVAL_Y_MAX nohead ls 4

set ylabel '$\rho(t)$'
set xlabel '\$W$'
set out '$PREFIX-auto.tex'
plot [:$PW] file i (( $k - 1 )) u 0:3:4 w yerr ls 1 t '$\rho(t)$', f(x) t 'Tail' ls 60 lw 4  

unset arrow
set key top right box spacing 1.2
set ylabel '$\tau_{\rm int}$'
set xlabel '\$W$'
set out '$PREFIX-taui.tex'
plot [:$PW] file i (( $k - 1 )) u 0:5:6 w yerr ls 1 t '$\tau_{\rm int}$' 
set arrow from $IW[k]+0.5,GPVAL_Y_MIN to $IW[k]+0.5,GPVAL_Y_MAX nohead ls 4
set object 1 rect from 0,$TI[k]-$DT[k] to $PW,$TI[k]+$DT[k] fc ls 60 back fillstyle solid 0.3 noborder 

set ylabel '$\tau_{\rm int}$'
set out '$PREFIX-taui.tex'
plot [:$PW] file i (( $k - 1 )) u 0:5:6 w yerr ls 1 t '$\tau_{\rm int}$' 

set term tikz standalone color font ",14" size 24,8
unset arrow
unset object
set key horizontal left
set size ratio 0.33
set ylabel ''
set xlabel 'MC time'
set out '$PREFIX-hist.tex'
filename(n) = sprintf("ID$k.gnp.%d", n)
plot [:$MS[k]]  for [i=1:$RP[k]] \
filename(i) u 1:2 w l lc i lw 1 t '' 


DELIM

else
    cat > $GNP <<DELIM
set colorsequence default
set style line 1 lw 2 lt 1 pt 7  ps 0.8
set style line 2 lw 2 lt 2 pt 7	 ps 0.8
set style line 3 lw 2 lt 7 pt 7	 ps 0.8
set style line 4 lw 2 lt -1 pt 7  ps 0.8
set style line 5 lw 2 lt 4 pt 7	 ps 0.8
set style line 6 lw 2 lt 18 pt 7 ps 0.8
set style line 60 lw 2 lt -1 pt 7 ps 0.8

set fit errorvariables
set term tikz standalone color font ",14"

file='$INPUT'

f(x) = x>$IW[k] ? $RHOVAL*exp(-(x - $IW[k])/$TX[k]) : NaN 

set key top right box spacing 1.2
set ylabel '$\rho(t)$'
set out '$PREFIX-auto.tex'
plot [:$PW] file i (( $k - 1 )) u 0:3:4 w yerr ls 1 t '$\rho(t)$'
set arrow from $IW[k]+0.5,GPVAL_Y_MIN to $IW[k]+0.5,GPVAL_Y_MAX nohead ls 4

set ylabel '$\rho(t)$'
set xlabel '\$W$'
set out '$PREFIX-auto.tex'
plot [:$PW] file i (( $k - 1 )) u 0:3:4 w yerr ls 1 t '$\rho(t)$', f(x) t 'Tail' ls 60 lw 4

unset arrow
set key top right box spacing 1.2
set ylabel '$\tau_{\rm int}$'
set xlabel '\$W$'
set out '$PREFIX-taui.tex'
plot [:$PW] file i (( $k - 1 )) u 0:5:6 w yerr ls 1 t '$\tau_{\rm int}$' 
set arrow from $IW[k]+0.5,GPVAL_Y_MIN to $IW[k]+0.5,GPVAL_Y_MAX nohead ls 4
set object 1 rect from 0,$TI[k]-$DT[k] to $PW,$TI[k]+$DT[k] fc ls 60 back fillstyle solid 0.3 noborder 

set ylabel '$\tau_{\rm int}$'
set out '$PREFIX-taui.tex'
plot [:$PW] file i (( $k - 1 )) u 0:5:6 w yerr ls 1 t '$\tau_{\rm int}$' 

set term tikz standalone color font ",14" size 24,8
unset arrow
unset object
set key horizontal left
set ylabel ''
set xlabel 'MC time'
set out '$PREFIX-hist.tex'
filename(n) = sprintf("ID$k.gnp.%d", n)
plot [:$MS[k]]  for [i=1:$RP[k]] \
filename(i) u 1:2 w l lc i lw 1 t "Replica ".i 


DELIM

    fi
    
    cd $DIR
    echo $GNP
 
    gnuplot ID$k.gnp
    pdflatex $PREFIX-hist.tex
    pdflatex $PREFIX-auto.tex
    pdflatex $PREFIX-taui.tex
    
    rm $PREFIX-hist.log $PREFIX-auto.log $PREFIX-taui.log
    rm $PREFIX-taui.aux $PREFIX-auto.aux $PREFIX-hist.aux
    rm $PREFIX-taui.tex $PREFIX-auto.tex $PREFIX-hist.tex
    cd ..
    
done

